@@ -36,7 +36,7 @@ from .utils import print_rank_0
 from .utils import get_sample_writer
 
 from SwissArmyTransformer import mpu
-from SwissArmyTransformer.data_utils import make_loaders 
+from SwissArmyTransformer.data_utils import make_loaders
 from SwissArmyTransformer.tokenization import get_tokenizer
 
 
@@ -46,23 +46,23 @@ def training_main(args, model_cls, forward_step_function, create_dataset_functio
         'forward_step': forward_step_function,
         'init_function': init_function,
         'create_dataset_function': create_dataset_function
-        }
-    
+    }
+
     torch.backends.cuda.matmul.allow_tf32 = False
-    torch.backends.cudnn.enabled = False # Disable CuDNN.
-    timers = Timers() # Timer.
-    
+    torch.backends.cudnn.enabled = False  # Disable CuDNN.
+    timers = Timers()  # Timer.
+
     # Experiment Name
-    if args.load and args.mode == 'pretrain': # continue training
+    if args.load and args.mode == 'pretrain':  # continue training
         args.experiment_name = os.path.basename(os.path.normpath(args.load))
     else:
         args.experiment_name = args.experiment_name + datetime.now().strftime("%m-%d-%H-%M")
-        
+
     # Pytorch distributed. must before seed
     initialize_distributed(args)
-    set_random_seed(args.seed) # Random seeds for reproducability.
+    set_random_seed(args.seed)  # Random seeds for reproducability.
     # init tokenizer
-    get_tokenizer(args) # set args.vocab_size.
+    get_tokenizer(args)  # set args.vocab_size.
     # Data stuff.
     train_data, val_data, test_data = make_loaders(args, hooks['create_dataset_function'])
 
@@ -80,7 +80,7 @@ def training_main(args, model_cls, forward_step_function, create_dataset_functio
     if args.save:
         args.save = os.path.join(args.save, args.experiment_name)
     torch.distributed.barrier()
-    
+
     # initialize lr scheduler
     lr_scheduler = get_learning_rate_scheduler(optimizer, args.iteration, args)
 
@@ -100,15 +100,7 @@ def training_main(args, model_cls, forward_step_function, create_dataset_functio
         if val_data is not None:
             start_iter_val = (args.train_iters // args.save_interval) * args.eval_interval
             val_data.batch_sampler.start_iter = start_iter_val % len(val_data)
-    if train_data is not None:
-        train_data_iterator = iter(train_data)
-    else:
-        train_data_iterator = None
-    if val_data is not None:
-        val_data_iterator = iter(val_data)
-    else:
-        val_data_iterator = None
-        
+
     # init hook before training
     if hooks['init_function'] is not None:
         hooks['init_function'](args, model, optimizer)
@@ -122,25 +114,22 @@ def training_main(args, model_cls, forward_step_function, create_dataset_functio
                     save_checkpoint(args_.iteration, model_, optimizer_, lr_scheduler_, args_)
                 iteration, skipped = train(model, optimizer,
                     lr_scheduler,
-                    train_data_iterator,
-                    val_data_iterator,
+                    train_data,
+                    val_data,
                     timers, args, summary_writer=summary_writer,
                     hooks=hooks
                     )
-        if args.do_valid:
-            prefix = 'the end of training for val data'
-            val_loss = evaluate_and_print_results(prefix, val_data_iterator,
-                model, args, timers, False)
 
     # final save
-    if args.save and iteration != 0: # TODO save
+    if args.save and iteration != 0:  # TODO save
         save_checkpoint(iteration, model, optimizer, lr_scheduler, args)
 
     # final testing
     if args.do_test and test_data is not None:
         prefix = 'the end of training for test data'
         evaluate_and_print_results(prefix, iter(test_data),
-            model, args, timers, True)
+            model, len(test_data) if args.strict_eval else args.eval_iters, args, timers, True, hooks=hooks)
+
 
 def get_model(args, model_cls):
     """Build the model."""
@@ -155,16 +144,19 @@ def get_model(args, model_cls):
 
     if args.fp16:
         model.half()
+    elif args.bf16:
+        model.bfloat16()
     model.cuda(torch.cuda.current_device())
 
     return model
 
-def setup_model_and_optimizer(args, model_cls):
+
+def setup_model_and_optimizer(args, model_cls, config_params=None):
     """Setup model and optimizer."""
 
     model = get_model(args, model_cls)
-    
-    model.disable_untrainable_params() # mark trainable params
+
+    model.disable_untrainable_params()  # mark trainable params
 
     param_groups = get_optimizer_param_groups(model)
 
@@ -176,7 +168,8 @@ def setup_model_and_optimizer(args, model_cls):
                 model_parameters=param_groups,
                 args=args,
                 mpu=mpu,
-                dist_init_required=False
+                dist_init_required=False,
+                config_params=config_params
             )
         else:
             raise ValueError('Currently, we only support training with deepspeed.')
@@ -187,7 +180,6 @@ def setup_model_and_optimizer(args, model_cls):
 
 
 def get_params_for_weight_decay_optimization(module):
-    
     weight_decay_params = {'params': []}
     no_weight_decay_params = {'params': [], 'weight_decay': 0.0}
     for module_ in module.modules():
@@ -210,11 +202,12 @@ def get_params_for_weight_decay_optimization(module):
 
     return weight_decay_params, no_weight_decay_params
 
+
 def get_optimizer_param_groups(model):
     # Build parameter groups (weight decay and non-decay).
     if hasattr(model, 'module'):
         model = model.module
-    param_groups = get_params_for_weight_decay_optimization(model) # TODO move to here
+    param_groups = get_params_for_weight_decay_optimization(model)  # TODO move to here
     # Add model parallel attribute if it is not set.
     for param_group in param_groups:
         for param in param_group['params']:
@@ -222,7 +215,8 @@ def get_optimizer_param_groups(model):
                 param.model_parallel = False
     return param_groups
 
-def get_learning_rate_scheduler(optimizer, iteration, args, 
+
+def get_learning_rate_scheduler(optimizer, iteration, args,
                                 auto_warmup_steps=100, auto_warmup_rate=0.05):
     """Build the learning rate scheduler."""
 
@@ -232,7 +226,7 @@ def get_learning_rate_scheduler(optimizer, iteration, args,
     else:
         num_iters = args.train_iters
     num_iters = max(1, num_iters)
-    init_step = max(iteration-auto_warmup_steps, 0)
+    init_step = max(iteration - auto_warmup_steps, 0)
     if args.mode == 'pretrain' and iteration == 0:
         auto_warmup_steps = 0
     # If init_step <= current_steps <= init_step + auto_warmup_steps,
@@ -240,25 +234,35 @@ def get_learning_rate_scheduler(optimizer, iteration, args,
     # This overrides other rules.
     warmup_iter = args.warmup * num_iters
     lr_scheduler = AnnealingLR(optimizer,
-        start_lr=args.lr,
-        warmup_iter=warmup_iter,
-        num_iters=num_iters,
-        decay_style=args.lr_decay_style,
-        last_iter=init_step,
-        decay_ratio=args.lr_decay_ratio,
-        auto_warmup_steps=auto_warmup_steps,
-        auto_warmup_rate=auto_warmup_rate
-        )
+                               start_lr=args.lr,
+                               warmup_iter=warmup_iter,
+                               num_iters=num_iters,
+                               decay_style=args.lr_decay_style,
+                               last_iter=init_step,
+                               decay_ratio=args.lr_decay_ratio,
+                               auto_warmup_steps=auto_warmup_steps,
+                               auto_warmup_rate=auto_warmup_rate
+                               )
 
     return lr_scheduler
 
 
 def train(model, optimizer, lr_scheduler,
-        train_data_iterator, val_data_iterator, timers, args, 
+        train_data, val_data, timers, args, 
         summary_writer=None, hooks={}):
     """Train the model."""
+    if train_data is not None:
+        train_data_iterator = iter(train_data)
+    else:
+        train_data_iterator = None
+    if val_data is not None:
+        val_data_iterator = iter(val_data)
+    else:
+        val_data_iterator = None
+        
     # Turn on training mode which enables dropout.
     model.train()
+    
 
     # Tracking loss.
     total_lm_loss = 0.0
@@ -272,10 +276,10 @@ def train(model, optimizer, lr_scheduler,
     while args.iteration < args.train_iters:
 
         lm_loss, skipped_iter, metrics = train_step(train_data_iterator,
-                                        model,
-                                        optimizer,
-                                        lr_scheduler,
-                                        args, timers, hooks=hooks)
+                                                    model,
+                                                    optimizer,
+                                                    lr_scheduler,
+                                                    args, timers, hooks=hooks)
         skipped_iters += skipped_iter
         args.iteration += 1
 
@@ -295,8 +299,8 @@ def train(model, optimizer, lr_scheduler,
 
             elapsed_time = timers('interval time').elapsed()
             report_iteration_metrics(summary_writer, optimizer, learning_rate, avg_lm_loss,
-                                    elapsed_time * 1000.0 / args.log_interval, args.iteration, args.train_iters, args,
-                                    avg_metrics)
+                                     elapsed_time * 1000.0 / args.log_interval, args.iteration, args.train_iters, args,
+                                     avg_metrics)
             total_lm_loss = 0.0
             total_metrics = defaultdict(float)
             if report_memory_flag:
@@ -304,17 +308,22 @@ def train(model, optimizer, lr_scheduler,
                 report_memory_flag = False
 
             timers.log(['forward', 'backward', 'allreduce', 'optimizer',
-                            'batch generator', 'data loader'],
-                        normalizer=args.log_interval)
+                        'batch generator', 'data loader'],
+                       normalizer=args.log_interval)
         # Checkpointing
         if args.save and args.save_interval and args.iteration % args.save_interval == 0:
             save_checkpoint(args.iteration, model, optimizer, lr_scheduler, args)
 
         # Evaluation
         if args.eval_interval and args.iteration % args.eval_interval == 0 and args.do_valid:
+            if args.strict_eval:
+                val_data_iterator = iter(val_data)
+                eval_iters = len(val_data)
+            else:
+                eval_iters = args.eval_iters
             prefix = 'iteration {}'.format(args.iteration)
             evaluate_and_print_results(
-                prefix, val_data_iterator, model, args, timers, False, step=args.iteration, summary_writer=summary_writer, hooks=hooks)
+                prefix, val_data_iterator, model, eval_iters, args, timers, False, step=args.iteration, summary_writer=summary_writer, hooks=hooks)
 
         if args.exit_interval and args.iteration % args.exit_interval == 0:
             torch.distributed.barrier()
@@ -328,14 +337,17 @@ def train(model, optimizer, lr_scheduler,
 
 
 def train_step(data_iterator, model, optimizer, lr_scheduler,
-               args, timers, hooks={}):
+               args, timers, hooks=None, single_step=False, **kwargs):
     """Single training step."""
+    if hooks is None:
+        hooks = {}
+    lm_loss_total, metrics_total, count = 0.0, {}, 0
     forward_step = hooks['forward_step']
-    
+
     while True:
         # Forward model for one step.
         timers('forward').start()
-        lm_loss, metrics = forward_step(data_iterator, model, args, timers)
+        lm_loss, metrics = forward_step(data_iterator, model, args, timers, **kwargs)
         timers('forward').stop()
 
         # Check nan or inf in forward, preventing it from interfering loss scaler,
@@ -354,6 +366,13 @@ def train_step(data_iterator, model, optimizer, lr_scheduler,
             print('Skipping backward and optimizer step for nan or inf in forwarding metrics/loss!')
             return lm_loss.detach(), 1, metrics
 
+        # Accumulate the statistics
+        lm_loss_total += lm_loss_reduced
+        for name in metrics:
+            if name not in metrics_total:
+                metrics_total[name] = 0.0
+            metrics_total[name] += metrics[name]
+        count += 1
         # Calculate gradients, reduce across processes, and clip.
         timers('backward').start()
         backward_step(optimizer, model, lm_loss, args, timers)
@@ -374,9 +393,12 @@ def train_step(data_iterator, model, optimizer, lr_scheduler,
         else:
             raise ValueError('Currently, we only support training with deepspeed.')
         timers('optimizer').stop()
-        if complete:
+        if complete or single_step:
             break
-    return lm_loss_reduced, skipped_iter, metrics
+    lm_loss_total /= count
+    metrics_total = {key: value / count for key, value in metrics_total.items()}
+    return lm_loss_total, skipped_iter, metrics_total
+
 
 def backward_step(optimizer, model, loss, args, timers):
     """Backward step."""
@@ -394,20 +416,19 @@ def backward_step(optimizer, model, loss, args, timers):
 
     return
 
-def evaluate(data_iterator, model, args, timers, verbose=False, hooks={}):
+def evaluate(data_iterator, model, eval_iters, args, timers, verbose=False, hooks={}):
     """Evaluation."""
     forward_step = hooks['forward_step']
-
     # Turn on evaluation mode which disables dropout.
     model.eval()
 
-    total_lm_loss = 0
+    total_lm_loss, metrics_total = 0, {}
     with torch.no_grad():
         iteration = 0
-        while iteration < args.eval_iters:
+        while iteration < eval_iters:
             iteration += 1
             if verbose and iteration % args.log_interval == 0:
-                print_rank_0('Evaluating iter {}/{}'.format(iteration, args.eval_iters))
+                print_rank_0('Evaluating iter {}/{}'.format(iteration, eval_iters))
             # Forward evaluation.
             lm_loss, metrics = forward_step(data_iterator, model, args, timers)
             '''when contiguous memory optimizations are enabled, the buffers
@@ -417,29 +438,35 @@ def evaluate(data_iterator, model, args, timers, verbose=False, hooks={}):
             if args.deepspeed and args.deepspeed_activation_checkpointing:
                 deepspeed.checkpointing.reset()
             total_lm_loss += lm_loss.data.detach().float().item()
+            for name in metrics:
+                if name not in metrics_total:
+                    metrics_total[name] = 0.0
+                metrics_total[name] += metrics[name]
 
     # Move model back to the train mode.
     model.train()
 
-    total_lm_loss /= args.eval_iters
-    return total_lm_loss
+    total_lm_loss /= eval_iters
+    metrics_avg = {key: value / eval_iters for key, value in metrics_total.items()}
+    return total_lm_loss, metrics_avg
 
-def evaluate_and_print_results(prefix, data_iterator, model,
+def evaluate_and_print_results(prefix, data_iterator, model, eval_iters,
                             args, timers, verbose=False, step=None, summary_writer=None, hooks={}):
     """Helper function to evaluate and dump results on screen."""
     # import line_profiler
     # profile = line_profiler.LineProfiler(model.module.module.transformer.layers[0].forward)
     # profile.enable()
     # torch.cuda.empty_cache()
-    lm_loss = evaluate(data_iterator, model, args, timers, verbose, hooks=hooks)
+    lm_loss, metrics = evaluate(data_iterator, model, eval_iters, args, timers, verbose, hooks=hooks)
     # profile.disable()
     # import sys
     # profile.print_stats(sys.stdout)
     lm_ppl = math.exp(min(20, lm_loss))
-    report_evaluate_metrics(summary_writer, prefix, lm_loss, lm_ppl, step)
+    report_evaluate_metrics(summary_writer, prefix, lm_loss, lm_ppl, step, metrics)
 
     return lm_loss
 
+
 def report_iteration_metrics(summary_writer, optimizer, lr, loss, elapsed_time, step, total_step, args, avg_metrics):
     log_string = ' iteration {:8d}/{:8d} |'.format(step, total_step)
     log_string += ' elapsed time per iteration (ms): {:.1f} |'.format(elapsed_time)
@@ -455,12 +482,16 @@ def report_iteration_metrics(summary_writer, optimizer, lr, loss, elapsed_time,
         summary_writer.add_scalar(f'Train/lr', lr, step)
         summary_writer.add_scalar(f'Train/train_loss', loss, step)
         summary_writer.add_scalar(f'Train/elapsed_time', elapsed_time, step)
+        for key in avg_metrics:
+            summary_writer.add_scalar('Train/'+key, avg_metrics[key], step)
 
 
-def report_evaluate_metrics(summary_writer, prefix, loss, ppl, step):
+def report_evaluate_metrics(summary_writer, prefix, loss, ppl, step, avg_metrics):
     string = ' validation loss at {} | '.format(prefix)
     string += 'LM loss: {:.6E} | '.format(loss)
     string += 'LM PPL: {:.6E}'.format(ppl)
+    for key in avg_metrics:
+        string += ' {} {:.6E} |'.format(key, avg_metrics[key])
     length = len(string) + 1
     print_rank_0('-' * 100)
     print_rank_0('-' * length)
@@ -469,6 +500,8 @@ def report_evaluate_metrics(summary_writer, prefix, loss, ppl, step):
     if summary_writer is not None:
         summary_writer.add_scalar(f'Train/valid_ppl', ppl, step)
         summary_writer.add_scalar(f'Train/valid_loss', loss, step)
+        for key in avg_metrics:
+            summary_writer.add_scalar('Train/valid_'+key, avg_metrics[key], step)
         
 
 '''
@@ -486,6 +519,7 @@ def report_evaluate_metrics(summary_writer, prefix, loss, ppl, step):
     This must be done before all the calls to mpu.model_parallel_cuda_manual_seed
     '''
 
+
 def set_deepspeed_activation_checkpointing(args):
     deepspeed.checkpointing.configure(mpu, deepspeed_config=args.deepspeed_config, num_checkpoints=args.num_layers)
     mpu.checkpoint = deepspeed.checkpointing.checkpoint
@@ -500,9 +534,9 @@ def initialize_distributed(args):
     torch.cuda.set_device(args.device)
     # Call the init process
     init_method = 'tcp://'
-    master_ip = os.getenv('MASTER_ADDR', 'localhost')
-    master_port = os.getenv('MASTER_PORT', '6000')
-    init_method += master_ip + ':' + master_port
+    args.master_ip = os.getenv('MASTER_ADDR', 'localhost')
+    args.master_port = os.getenv('MASTER_PORT', '6000')
+    init_method += args.master_ip + ':' + args.master_port
     torch.distributed.init_process_group(
         backend=args.distributed_backend,
         world_size=args.world_size, rank=args.rank,
@@ -513,8 +547,9 @@ def initialize_distributed(args):
 
     # Optional DeepSpeed Activation Checkpointing Features
     if hasattr(args, "deepspeed") and args.deepspeed and args.deepspeed_activation_checkpointing:
-        set_deepspeed_activation_checkpointing(args) # TODO manual model-parallel seed
-
+        set_deepspeed_activation_checkpointing(args)  # TODO manual model-parallel seed
+    else:
+        mpu.get_cuda_rng_tracker = None
 
 def set_random_seed(seed):
     """Set random seed for reproducability."""
@@ -523,14 +558,10 @@ def set_random_seed(seed):
         np.random.seed(seed)
         torch.manual_seed(seed)
         torch.cuda.manual_seed(seed)
-        torch.cuda.manual_seed_all(seed) # if you are using multi-GPU.
+        torch.cuda.manual_seed_all(seed)  # if you are using multi-GPU.
         torch.backends.cudnn.benchmark = False
         torch.backends.cudnn.deterministic = True
         torch.backends.cudnn.enabled = False
         torch.backends.cuda.matmul.allow_tf32 = False
         if hasattr(mpu, 'model_parallel_cuda_manual_seed'):
             mpu.model_parallel_cuda_manual_seed(seed)
-        
-    
-
-
